package dev.shadowsoffire.apotheosis.adventure.socket.gem;

import net.minecraftforge.common.crafting.conditions.ICondition;
import java.util.Iterator;
import com.google.common.base.Preconditions;
import dev.shadowsoffire.apotheosis.adventure.loot.RarityRegistry;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus.GemBonus;
import dev.shadowsoffire.placebo.codec.CodecProvider;
import net.minecraft.resources.ResourceLocation;
import dev.shadowsoffire.apotheosis.adventure.affix.AffixHelper;
import dev.shadowsoffire.apotheosis.adventure.Adventure;
import net.minecraft.world.level.ItemLike;
import dev.shadowsoffire.apotheosis.adventure.loot.LootRarity;
import dev.shadowsoffire.apotheosis.adventure.AdventureConfig;
import dev.shadowsoffire.apotheosis.adventure.loot.RarityClamp;
import net.minecraft.world.item.ItemStack;
import java.util.function.Predicate;
import net.minecraft.server.level.ServerLevel;
import net.minecraft.util.RandomSource;
import com.mojang.serialization.Codec;
import dev.shadowsoffire.apotheosis.Apotheosis;
import dev.shadowsoffire.apotheosis.adventure.AdventureModule;
import dev.shadowsoffire.placebo.reload.WeightedDynamicRegistry;

public class GemRegistry extends WeightedDynamicRegistry<Gem>
{
    public static final GemRegistry INSTANCE;
    
    public GemRegistry() {
        super(AdventureModule.LOGGER, "gems", true, false);
    }
    
    protected void registerBuiltinCodecs() {
        this.registerDefaultCodec(Apotheosis.loc("gem"), (Codec)Gem.CODEC);
    }
    
    @SafeVarargs
    public static ItemStack createRandomGemStack(final RandomSource rand, final ServerLevel level, final float luck, final Predicate<Gem>... filter) {
        final Gem gem = (Gem)GemRegistry.INSTANCE.getRandomItem(rand, luck, (Predicate[])filter);
        if (gem == null) {
            return ItemStack.f_41583_;
        }
        final RarityClamp clamp = AdventureConfig.GEM_DIM_RARITIES.get(level.m_46472_().m_135782_());
        final LootRarity rarity = gem.clamp(LootRarity.random(rand, luck, clamp));
        return createGemStack(gem, rarity);
    }
    
    public static ItemStack createGemStack(final Gem gem, final LootRarity rarity) {
        final ItemStack stack = new ItemStack((ItemLike)Adventure.Items.GEM.get());
        GemItem.setGem(stack, gem);
        AffixHelper.setRarity(stack, rarity);
        return stack;
    }
    
    protected void validateItem(final ResourceLocation key, final Gem item) {
        super.validateItem(key, (CodecProvider)item);
        for (LootRarity r = item.minRarity; r != item.maxRarity; r = r.next()) {
            boolean atLeastOne = false;
            for (final GemBonus bonus : item.bonuses) {
                if (bonus.supports(r)) {
                    atLeastOne = true;
                }
            }
            Preconditions.checkArgument(atLeastOne, "No bonuses provided for supported rarity %s. At least one bonus must be provided, or the rarity should not be supported.", (Object)RarityRegistry.INSTANCE.getKey((CodecProvider)r));
        }
    }
    
    public final ICondition.IContext _getContext() {
        return this.getContext();
    }
    
    static {
        INSTANCE = new GemRegistry();
    }
}
